Package: gCrisprTools
Type: Package
Title: Suite of Functions for Pooled Crispr Screen QC and Analysis
Version: 2.16.0
Date: 2021-08-23
Author: Russell Bainer, Dariusz Ratman, Steve Lianoglou, Peter Haverty
Authors@R: c(
    person("Russell", "Bainer", , "russ.bainer@gmail.com", c("aut", "cre")),
    person("Dariusz", "Ratman", , "ratmand@roche.com", "aut"),
    person("Peter", "Haverty", , "havertypm@gmail.com", "aut"),
    person("Steve", "Lianoglou", , "slianoglou@gmail.com", "aut"),
    person("Maze Therapeutics", role = c("fnd"), comment = "2019+"),
    person("Genentech, Inc.", role = c("fnd"), comment = "2014 - 2018"))
Maintainer: Russell Bainer <russ.bainer@gmail.com>
Description: Set of tools for evaluating pooled high-throughput screening experiments,
    typically employing CRISPR/Cas9 or shRNA expression cassettes. Contains methods for interrogating
    library and cassette behavior within an experiment, identifying differentially abundant
    cassettes, aggregating signals to identify candidate targets for empirical validation,
    hypothesis testing, and comprehensive reporting. Version 2.0 extends these applications to include 
    a variety of tools for contextualizing and integrating signals across many experiments, incorporates 
    extended signal enrichment methodologies via the "sparrow" package, and 
    streamlines many formal requirements to aid in interpretablity. 
License: Artistic-2.0
Imports: Biobase, limma, RobustRankAggreg, ggplot2,
        SummarizedExperiment, grid, rmarkdown, grDevices, graphics,
        methods, ComplexHeatmap, stats, utils, parallel
Suggests: edgeR, knitr, AnnotationDbi, org.Mm.eg.db, org.Hs.eg.db,
        BiocGenerics, markdown, RUnit, sparrow, msigdbr, fgsea
RoxygenNote: 7.2.1
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: ImmunoOncology, CRISPR, PooledScreens, ExperimentalDesign,
        BiomedicalInformatics, CellBiology, FunctionalGenomics,
        Pharmacogenomics, Pharmacogenetics, SystemsBiology,
        DifferentialExpression, GeneSetEnrichment, Genetics,
        MultipleComparison, Normalization, Preprocessing,
        QualityControl, RNASeq, Regression, Software, Visualization
NeedsCompilation: no
Depends: R (>= 4.1)
Packaged: 2025-10-30 04:06:32 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/gCrisprTools
git_branch: RELEASE_3_22
git_last_commit: 7b3b1fd
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
