# gCrisprTools
An R/Bioconductor package for the integrated analysis of pooled high-throughput CRISPR/Cas9 or shRNA screening experiments.

The initial version of this package was created at Genentech, inc. Current maintenance, development, and extension is 
supervised by the team at Maze Therapeutics.

See vignettes for examples of screen design, processing, analysis, and integration. 
