% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gCrisprTools.R
\docType{package}
\name{gCrisprTools-package}
\alias{gCrisprTools-package}
\title{gCrisprTools}
\description{
Pipeline for using CRISPR screen data
}
