% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ViewGuides.R
\name{ct.exprsColor}
\alias{ct.exprsColor}
\title{Assign Colors Based on the Position of a Value in a Distribution}
\usage{
ct.exprsColor(exprs, rankedexprs, colors)
}
\arguments{
\item{exprs}{The value whose color is to be returned.}

\item{rankedexprs}{A vector of values the length of cols that corresponds to the values in the distribution.}

\item{colors}{The vector of colors to be used as a reference.}
}
\value{
A value contained in cols.
}
\description{
This is a function to generate colors for plot elements on the basis of the position of a value within a distribution. 
Called internally by ct.viewGuides.
}
\author{
Russell Bainer
}
\keyword{internal}
