% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ViewGuides.R
\name{ct.drawFlat}
\alias{ct.drawFlat}
\title{Draw a horizontal line of a specified color.}
\usage{
ct.drawFlat(x, y, color, width = 1)
}
\arguments{
\item{x, y}{Minimal coordinates to specify the line, which is drawn from the Y axis.}

\item{color}{Guess!}

\item{width}{Line width.}
}
\value{
A line on an open device.
}
\description{
This is a function called internally by ct.viewGuides to generate the color legend. End users should not use it.
}
\author{
Russell Bainer
}
\keyword{internal}
